import 'package:ocg_app/ocg_app.dart';
import 'package:bloc/bloc.dart';
import 'package:ocg_login/src/repository/login/base.dart';

import 'event.dart';
import 'state.dart';

class LoginBloc extends Bloc<LoginEvent, LoginState> {
  final LoginRepository repository;
  TokenRepository tokenRepository;
  LoginBloc(
    this.repository,
    this.tokenRepository,
  ) : super(LoginIdleState());

  @override
  Stream<LoginState> mapEventToState(LoginEvent event) async* {
    if (event is LoginCheckEvent) {
      AppResponse<AppToken> tokenResponse = await tokenRepository.getToken();
      if (tokenResponse.isSuccess) {
        yield LoginSuccessState(tokenResponse.data);
      } else {
        yield LoginErrorState(LoginError.NOT_EXISTS);
      }
    }

    if (event is LoginMobileEvent) {
      yield LoginLoadingState();

      var response = await repository.sendOTP(event.mobileNumber);
      if (response.isSuccess)
        yield LoginOTPNotVerifiedState(event.mobileNumber);
      else {
        yield LoginErrorState(response.error);
      }
    }
    if (event is LoginOTPEvent) {
      yield LoginLoadingState();
      var response = await repository.verifyOTP(event.mobileNumber, event.otp);
      if (response.isSuccess) {
        yield LoginSuccessState(response.data);
      } else {
        yield LoginErrorState(response.error);
      }
    }
    if (event is LoginEmailSignupEvent) {
      yield LoginLoadingState();
      var response = await repository.createUserWithEmailAndPassword(
          event.email, event.password);
      if (response.isSuccess) {
        yield LoginSuccessState(response.data);
      } else {
        yield LoginErrorState(response.error);
      }
    }
    if (event is LoginEmailSigninEvent) {
      yield LoginLoadingState();
      var response = await repository.signInWithEmailAndPassword(
          event.email, event.password);
      if (response.isSuccess) {
        yield LoginSuccessState(response.data);
      } else {
        yield LoginErrorState(response.error);
      }
    }
    if (event is LoginGoogleSigninEvent) {
      yield LoginLoadingState();
      var response = await repository.signWithGoogle();
      if (response.isSuccess) {
        yield LoginSuccessState(response.data);
      } else {
        yield LoginErrorState(response.error);
      }
    }
    if (event is LoginAppleSigninEvent) {
      yield LoginLoadingState();
      var response = await repository.signInWithApple();
      if (response.isSuccess) {
        yield LoginSuccessState(response.data);
      } else {
        yield LoginErrorState(response.error);
      }
    }
    if (event is LoginFacebookSigninEvent) {
      yield LoginLoadingState();
      var response = await repository.signInWithFacebook();
      if (response.isSuccess) {
        yield LoginSuccessState(response.data);
      } else {
        yield LoginErrorState(response.error);
      }
    }
  }
}
